import subprocess
import logging
import os
import signal


logger = logging.getLogger(__name__)


def _rds_cmd_options(config):
    return [
        '--region',
        config.region,
        '--access-key-id',
        config.access_key,
        '--secret-key',
        config.secret_key,
    ]


class RDSLogDownload:
    def __init__(self, config, local_file):
        self._config = config
        self._local_file = local_file

    def download(self):
        cmd = [
            'rds',
            'rds-download-db-logfile',
            self._config.db_identifier,
            '--log-file-name',
            self._local_file.filename,
        ] + _rds_cmd_options(self._config)

        with self._local_file.open_for_write() as f:
            self._proc = subprocess.Popen(cmd, stdout=f)
            return_code = self._proc.wait()

            if return_code == 0:
                logger.info('Download success')
            else:
                logger.warning('Download failed. rds-download-db-logfile exit code {}'.format(return_code))

    def __repr__(self):
        return '<RDSLogDownLoad "{}">'.format(self._local_file.filename)


class RDSLogStream:
    def __init__(self, config, local_file):
        self._config = config
        self._local_file = local_file

    def __eq__(self, other):
        return hasattr(other, '_local_file') and other._local_file.filename == self._local_file.filename

    def start_stream(self):
        cmd = [
            'rds',
            'rds-watch-db-logfile',
            self._config.db_identifier,
            '--log-file-name',
            self._local_file.filename,
        ] + _rds_cmd_options(self._config)

        logger.info('Starting stream of {}'.format(self._local_file.filename))

        self._out_file = self._local_file.open_for_write()
        self._proc = subprocess.Popen(cmd,
                                      stdout=self._out_file,
                                      start_new_session=True)

    def stop_stream(self):
        logger.info('Stopping stream of {}'.format(self._local_file.filename))
        os.killpg(self._proc.pid, signal.SIGKILL)
        self._out_file.close()

    def __repr__(self):
        return '<RDSLogStream "{}">'.format(self._local_file.filename)




class LocalLogFile:
    def __init__(self, root_directory, filename):
        self.filename = filename
        self._root_directory = root_directory
        self._path = self._root_directory / self.filename

    @property
    def size(self):
        try:
            return self._path.stat().st_size
        except FileNotFoundError:
            return -1

    def open_for_write(self):

        try:
            self._path.parent.mkdir(parents=True)
        except FileExistsError:
            pass

        return self._path.open('w+b')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import codecs
from setuptools import setup, find_packages


# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return codecs.open(file_path, encoding='utf-8').read()


setup(
    name='rds-log',
    use_scm_version=True,
    description='A utility to download AWS RDS logs',
    author='Andreas Pelme',
    author_email='andreas@pelme.se',
    url='https://github.com/pelme/rds-log',
    packages=find_packages(),
    long_description=read('README.rst'),
    install_requires=[
        'boto3==1.1.2',
        'click==5.1',
    ],
    setup_requires=[
        'setuptools_scm==1.7.0',
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python :: 3.4',
    ],
    entry_points={
        'console_scripts': 'rds-log-stream=_rds_log.commands.rds_log_stream:main'
    },
)



import sys
import click
import pathlib
import time
import operator
import logging

from collections import namedtuple

import boto3

logging.basicConfig(stream=sys.stderr, level=logging.INFO)
logger = logging.getLogger(__name__)


from ..rds import RDSLogDownload, RDSLogStream
from ..local_log import LocalLogFile


def _get_log_files(config, db_identifier, root_directory):
    result = config.rds_client.describe_db_log_files(DBInstanceIdentifier=db_identifier)

    rds_logs = sorted((rds_log for rds_log in result['DescribeDBLogFiles']),
                      key=operator.itemgetter('LogFileName'))

    if not rds_logs:
        logger.info('No RDS logs was retrieved: {}'.format(rds_logs))

    def downloads():
        for rds_log in rds_logs[:-1]:
            local_file = LocalLogFile(root_directory, rds_log['LogFileName'])

            if local_file.size != rds_log['Size']:
                yield RDSLogDownload(config, local_file)

    if rds_logs:
        stream = RDSLogStream(config, LocalLogFile(root_directory, rds_logs[-1]['LogFileName']))
    else:
        stream = None

    return (list(downloads()), stream)


_Config = namedtuple('Config', ['access_key', 'secret_key', 'region', 'db_identifier', 'rds_client'])


@click.command()
@click.version_option()
@click.argument('db_identifier')
@click.argument('destination_directory', type=click.Path(file_okay=False, dir_okay=True, exists=True))
def main(db_identifier, destination_directory):
    destination_directory = pathlib.Path(destination_directory)
    current_stream = None

    boto_session = boto3.session.Session()

    config = _Config(
        access_key=boto_session._session.get_credentials().access_key,
        secret_key=boto_session._session.get_credentials().secret_key,
        region=boto_session._session.get_config_variable('region'),
        db_identifier=db_identifier,
        rds_client=boto_session.client('rds'),
    )

    try:

        while True:
            downloads, stream = _get_log_files(config, db_identifier, destination_directory)
            stream_changed = stream != current_stream

            if stream_changed:
                if current_stream is not None:
                    current_stream.stop_stream()

                for download in downloads:
                    logger.info('Downloading full copy of {}'.format(download))
                    download.download()

                current_stream = stream

                if current_stream is not None:
                    current_stream.start_stream()

            else:
                logger.info('Check completed, no new file found')

            time.sleep(60)

    finally:
        if current_stream:
            current_stream.stop_stream()
